@@ -25,7 +25,6 @@ import netifaces
 from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
-import six
 
 from nova.accelerator import cyborg
 from nova import block_device
@@ -69,6 +68,13 @@ NON_INHERITABLE_IMAGE_PROPERTIES = frozenset([
     'img_signature_key_type',
     'img_signature_certificate_uuid'])
 
+# Properties starting with these namespaces are reserved for internal
+# use by other services. It does not make sense (and may cause a request
+# fail) if we include them in a snapshot.
+NON_INHERITABLE_IMAGE_NAMESPACES = frozenset([
+    'os_glance',
+])
+
 
 def exception_to_dict(fault, message=None):
     """Converts exceptions to a dict for use in notifications.
@@ -131,7 +137,7 @@ def _get_fault_details(exc_info, error_code):
         # exceptions (see exception_to_dict).
         details = ''.join(traceback.format_exception(
             exc_info[0], exc_info[1], exc_info[2]))
-    return six.text_type(details)
+    return str(details)
 
 
 def add_instance_fault_from_exc(context, instance, fault, exc_info=None,
@@ -356,9 +362,6 @@ def notify_usage_exists(notifier, context, instance_ref, host,
 
     audit_start, audit_end = notifications.audit_period_bounds(current_period)
 
-    bw = notifications.bandwidth_usage(context, instance_ref, audit_start,
-            ignore_missing_network_data)
-
     if system_metadata is None:
         system_metadata = utils.instance_sys_meta(instance_ref)
 
@@ -367,7 +370,7 @@ def notify_usage_exists(notifier, context, instance_ref, host,
 
     extra_info = dict(audit_period_beginning=str(audit_start),
                       audit_period_ending=str(audit_end),
-                      bandwidth=bw, image_meta=image_meta)
+                      image_meta=image_meta)
 
     if extra_usage_info:
         extra_info.update(extra_usage_info)
@@ -376,36 +379,34 @@ def notify_usage_exists(notifier, context, instance_ref, host,
                                 extra_usage_info=extra_info)
 
     audit_period = instance_notification.AuditPeriodPayload(
-            audit_period_beginning=audit_start,
-            audit_period_ending=audit_end)
-
-    bandwidth = [instance_notification.BandwidthPayload(
-                    network_name=label,
-                    in_bytes=b['bw_in'],
-                    out_bytes=b['bw_out'])
-                 for label, b in bw.items()]
+        audit_period_beginning=audit_start,
+        audit_period_ending=audit_end,
+    )
 
     payload = instance_notification.InstanceExistsPayload(
         context=context,
         instance=instance_ref,
         audit_period=audit_period,
-        bandwidth=bandwidth)
+    )
 
     notification = instance_notification.InstanceExistsNotification(
         context=context,
         priority=fields.NotificationPriority.INFO,
         publisher=notification_base.NotificationPublisher(
-            host=host, source=fields.NotificationSource.COMPUTE),
+            host=host, source=fields.NotificationSource.COMPUTE,
+        ),
         event_type=notification_base.EventType(
             object='instance',
-            action=fields.NotificationAction.EXISTS),
-        payload=payload)
+            action=fields.NotificationAction.EXISTS,
+        ),
+        payload=payload,
+    )
     notification.emit(context)
 
 
 def notify_about_instance_usage(notifier, context, instance, event_suffix,
                                 network_info=None, extra_usage_info=None,
-                                fault=None):
+                                fault=None, best_effort=False):
     """Send an unversioned legacy notification about an instance.
 
     All new notifications should use notify_about_instance_action which sends
@@ -434,7 +435,14 @@ def notify_about_instance_usage(notifier, context, instance, event_suffix,
     else:
         method = notifier.info
 
-    method(context, 'compute.instance.%s' % event_suffix, usage_info)
+    try:
+        method(context, 'compute.instance.%s' % event_suffix, usage_info)
+    except Exception as e:
+        if best_effort:
+            LOG.error('Exception during notification sending: %s. '
+                      'Attempting to proceed with normal operation.', e)
+        else:
+            raise e
 
 
 def _get_fault_and_priority_from_exception(exception: Exception):
@@ -453,7 +461,7 @@ def _get_fault_and_priority_from_exception(exception: Exception):
 @rpc.if_notifications_enabled
 def notify_about_instance_action(context, instance, host, action, phase=None,
                                  source=fields.NotificationSource.COMPUTE,
-                                 exception=None, bdms=None):
+                                 exception=None, bdms=None, best_effort=False):
     """Send versioned notification about the action made on the instance
     :param instance: the instance which the action performed on
     :param host: the host emitting the notification
@@ -480,7 +488,14 @@ def notify_about_instance_action(context, instance, host, action, phase=None,
                     action=action,
                     phase=phase),
             payload=payload)
-    notification.emit(context)
+    try:
+        notification.emit(context)
+    except Exception as e:
+        if best_effort:
+            LOG.error('Exception during notification sending: %s. '
+                      'Attempting to proceed with normal operation.', e)
+        else:
+            raise e
 
 
 @rpc.if_notifications_enabled
@@ -1086,9 +1101,10 @@ def get_headroom(quotas, usages, deltas):
     return headroom
 
 
-def check_num_instances_quota(context, instance_type, min_count,
-                              max_count, project_id=None, user_id=None,
-                              orig_num_req=None):
+def check_num_instances_quota(
+    context, flavor, min_count, max_count, project_id=None, user_id=None,
+    orig_num_req=None,
+):
     """Enforce quota limits on number of instances created."""
     # project_id is also used for the TooManyInstances error message
     if project_id is None:
@@ -1103,11 +1119,13 @@ def check_num_instances_quota(context, instance_type, min_count,
     if not any(r in user_quotas for r in ['instances', 'cores', 'ram']):
         user_id = None
     # Determine requested cores and ram
-    req_cores = max_count * instance_type.vcpus
-    req_ram = max_count * instance_type.memory_mb
+    req_cores = max_count * flavor.vcpus
+    req_ram = max_count * flavor.memory_mb
     deltas = {'instances': max_count, 'cores': req_cores, 'ram': req_ram}
 
     try:
+        # NOTE(johngarbutt) when using unified limits, this is call
+        # is a no op, and as such, this function always returns max_count
         objects.Quotas.check_deltas(context, deltas,
                                     project_id, user_id=user_id,
                                     check_project_id=project_id,
@@ -1120,8 +1138,8 @@ def check_num_instances_quota(context, instance_type, min_count,
         if min_count == max_count == 0:
             # orig_num_req is the original number of instances requested in the
             # case of a recheck quota, for use in the over quota exception.
-            req_cores = orig_num_req * instance_type.vcpus
-            req_ram = orig_num_req * instance_type.memory_mb
+            req_cores = orig_num_req * flavor.vcpus
+            req_ram = orig_num_req * flavor.memory_mb
             requested = {'instances': orig_num_req, 'cores': req_cores,
                          'ram': req_ram}
             (overs, reqs, total_alloweds, useds) = get_over_quota_detail(
@@ -1139,21 +1157,19 @@ def check_num_instances_quota(context, instance_type, min_count,
 
         allowed = headroom.get('instances', 1)
         # Reduce 'allowed' instances in line with the cores & ram headroom
-        if instance_type.vcpus:
-            allowed = min(allowed,
-                          headroom['cores'] // instance_type.vcpus)
-        if instance_type.memory_mb:
-            allowed = min(allowed,
-                          headroom['ram'] // instance_type.memory_mb)
+        if flavor.vcpus:
+            allowed = min(allowed, headroom['cores'] // flavor.vcpus)
+        if flavor.memory_mb:
+            allowed = min(allowed, headroom['ram'] // flavor.memory_mb)
 
         # Convert to the appropriate exception message
         if allowed <= 0:
             msg = "Cannot run any more instances of this type."
         elif min_count <= allowed <= max_count:
             # We're actually OK, but still need to check against allowed
-            return check_num_instances_quota(context, instance_type, min_count,
-                                             allowed, project_id=project_id,
-                                             user_id=user_id)
+            return check_num_instances_quota(
+                context, flavor, min_count, allowed, project_id=project_id,
+                user_id=user_id)
         else:
             msg = "Can only run %s more instances of this type." % allowed
 
@@ -1278,6 +1294,9 @@ def initialize_instance_snapshot_metadata(context, instance, name,
     properties = image_meta['properties']
     keys_to_pop = set(CONF.non_inheritable_image_properties).union(
         NON_INHERITABLE_IMAGE_PROPERTIES)
+    for ns in NON_INHERITABLE_IMAGE_NAMESPACES:
+        keys_to_pop |= {key for key in properties
+                        if key.startswith(ns)}
     for key in keys_to_pop:
         properties.pop(key, None)
 
@@ -1486,14 +1505,14 @@ def notify_about_instance_delete(notifier, context, instance,
                 phase=fields.NotificationPhase.END)
 
 
-def update_pci_request_spec_with_allocated_interface_name(
-        context, report_client, instance, provider_mapping):
+def update_pci_request_with_placement_allocations(
+        context, report_client, pci_requests, provider_mapping):
     """Update the instance's PCI request based on the request group -
     resource provider mapping and the device RP name from placement.
 
     :param context: the request context
     :param report_client: a SchedulerReportClient instance
-    :param instance: an Instance object to be updated
+    :param pci_requests: A list of InstancePCIRequest objects to be updated
     :param provider_mapping: the request group - resource provider mapping
         in the form returned by the RequestSpec.get_request_group_mapping()
         call.
@@ -1504,15 +1523,36 @@ def update_pci_request_spec_with_allocated_interface_name(
         have a well formatted name so we cannot parse the parent interface
         name out of it.
     """
-    if not instance.pci_requests:
+    if not pci_requests:
         return
 
-    def needs_update(pci_request, mapping):
+    def needs_update_due_to_qos(pci_request, mapping):
         return (pci_request.requester_id and
                 pci_request.requester_id in mapping)
 
-    for pci_request in instance.pci_requests.requests:
-        if needs_update(pci_request, provider_mapping):
+    def get_group_mapping_for_flavor_based_pci_request(pci_request, mapping):
+        # NOTE(gibi): for flavor based PCI requests nova generates RequestGroup
+        # suffixes from InstancePCIRequests in the form of
+        # {request_id}-{count_index}
+        # NOTE(gibi): a suffixed request group always fulfilled from a single
+        # RP
+        return {
+            group_id: rp_uuids[0]
+            for group_id, rp_uuids in mapping.items()
+            if group_id.startswith(pci_request.request_id)
+        }
+
+    for pci_request in pci_requests:
+        mapping = get_group_mapping_for_flavor_based_pci_request(
+            pci_request, provider_mapping)
+
+        if mapping:
+            for spec in pci_request.spec:
+                # FIXME(gibi): this is baaad but spec is a dict of strings so
+                #  we need to serialize
+                spec['rp_uuids'] = ','.join(mapping.values())
+
+        elif needs_update_due_to_qos(pci_request, provider_mapping):
 
             provider_uuids = provider_mapping[pci_request.requester_id]
             if len(provider_uuids) != 1:
@@ -1538,16 +1578,28 @@ def update_pci_request_spec_with_allocated_interface_name(
                 spec['parent_ifname'] = rp_name_pieces[2]
 
 
-def delete_arqs_if_needed(context, instance):
-    """Delete Cyborg ARQs for the instance."""
-    dp_name = instance.flavor.extra_specs.get('accel:device_profile')
-    if dp_name is None:
-        return
+def delete_arqs_if_needed(context, instance, arq_uuids=None):
+    """Delete Cyborg ARQs for the instance.
+
+    :param context
+    :param instance: instance who own the args
+    :param uuids: delete arqs by uuids while did not bind to instance yet.
+    """
     cyclient = cyborg.get_client(context)
-    LOG.debug('Calling Cyborg to delete ARQs for instance %(instance)s',
+    dp_name = instance.flavor.extra_specs.get('accel:device_profile')
+
+    if dp_name:
+        LOG.debug('Calling Cyborg to delete ARQs for instance %(instance)s',
               {'instance': instance.uuid})
-    try:
-        cyclient.delete_arqs_for_instance(instance.uuid)
-    except exception.AcceleratorRequestOpFailed as e:
-        LOG.exception('Failed to delete accelerator requests for '
+        try:
+            cyclient.delete_arqs_for_instance(instance.uuid)
+        except exception.AcceleratorRequestOpFailed as e:
+            LOG.exception('Failed to delete accelerator requests for '
                       'instance %s. Exception: %s', instance.uuid, e)
+
+    if arq_uuids:
+        LOG.debug('Calling Cyborg to delete ARQs by uuids %(uuid)s for'
+              ' instance %(instance)s',
+              {'instance': instance.uuid,
+               'uuid': arq_uuids})
+        cyclient.delete_arqs_by_uuid(arq_uuids)
