@@ -14,9 +14,9 @@
 #    under the License.
 
 import copy
+from unittest import mock
 
 import fixtures
-import mock
 from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import block_device
@@ -42,6 +42,12 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         self.project_id = 'fake'
         self.context = context.get_admin_context()
         self.useFixture(nova_fixtures.GlanceFixture(self))
+
+        flavor = objects.Flavor(
+            id=2, name='m1.micro', vcpus=1, memory_mb=128, root_gb=0,
+            ephemeral_gb=0, swap=0, rxtx_factor=1.0, flavorid='1',
+            vcpu_weight=None,)
+
         self.test_instance = {
             'uuid': '32dfcb37-5af1-552b-357c-be8c3aa38310',
             'memory_kb': '1024000',
@@ -53,7 +59,10 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             'image_ref': '155d900f-4e14-4e4c-a73d-069cbf4541e6',
             'root_gb': 10,
             'ephemeral_gb': 20,
-            'instance_type_id': 2,  # m1.tiny
+            'instance_type_id': flavor.id,
+            'flavor': flavor,
+            'old_flavor': None,
+            'new_flavor': None,
             'config_drive': None,
             'launched_at': None,
             'system_metadata': {},
@@ -62,23 +71,10 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             'disk_format': 'raw',
         }
 
-        flavor = objects.Flavor(memory_mb=128,
-                                root_gb=0,
-                                name='m1.micro',
-                                ephemeral_gb=0,
-                                vcpus=1,
-                                swap=0,
-                                rxtx_factor=1.0,
-                                flavorid='1',
-                                vcpu_weight=None,
-                                id=2)
-        self.test_instance['flavor'] = flavor
-        self.test_instance['old_flavor'] = None
-        self.test_instance['new_flavor'] = None
-
     def _test_block_device_info(self, with_eph=True, with_swap=True,
                                 with_bdms=True):
         swap = {'device_name': '/dev/vdb', 'swap_size': 1}
+        image = [{'device_type': 'disk', 'boot_index': 0}]
         ephemerals = [{'device_type': 'disk', 'guest_format': 'ext4',
                        'device_name': '/dev/vdc1', 'size': 10},
                       {'disk_bus': 'ide', 'guest_format': None,
@@ -89,6 +85,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                  'device_path': 'fake_device'}]
         return {'root_device_name': '/dev/vda',
                 'swap': swap if with_swap else {},
+                'image': image,
                 'ephemerals': ephemerals if with_eph else [],
                 'block_device_mapping':
                     block_device_mapping if with_bdms else []}
@@ -183,11 +180,16 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
+        block_device_info = {
+            'image': [
+                {'device_type': 'disk', 'boot_index': 0},
+            ]
+        }
         with mock.patch.object(instance_ref, 'get_flavor',
                                return_value=instance_ref.flavor) as get_flavor:
-            mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
-                                                 "virtio", "ide",
-                                                 image_meta)
+            mapping = blockinfo.get_disk_mapping(
+                "kvm", instance_ref, "virtio", "ide", image_meta,
+                block_device_info=block_device_info)
         # Since there was no block_device_info passed to get_disk_mapping we
         # expect to get the swap info from the flavor in the instance.
         get_flavor.assert_called_once_with()
@@ -207,7 +209,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
         block_device_info = {
-            'root_device_name': '/dev/sda'
+            'root_device_name': '/dev/sda',
+            'image': [{'device_type': 'disk', 'boot_index': 0}],
             }
 
         mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
@@ -243,7 +246,9 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             }
         self.assertEqual(expect, mapping)
 
-    def test_get_disk_mapping_rescue_with_config(self):
+    def _test_get_disk_mapping_rescue_with_config(
+        self, expect_disk_config_rescue,
+    ):
         # A simple disk mapping setup, but in rescue mode with a config drive
 
         test_instance_with_config = self.test_instance
@@ -256,21 +261,34 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                              image_meta,
                                              rescue=True)
 
-        expect_disk_config_rescue = {
-            'bus': 'ide', 'dev': 'hda', 'type': 'cdrom'}
-        if blockinfo.libvirt_utils.get_arch({}) == 'aarch64':
-            expect_disk_config_rescue['bus'] = 'scsi'
-            expect_disk_config_rescue['dev'] = 'sda'
         expect = {
-            'disk.rescue': {'bus': 'virtio', 'dev': 'vda',
-                            'type': 'disk', 'boot_index': '1'},
+            'disk.rescue': {
+                'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk', 'boot_index': '1',
+            },
             'disk': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
             'disk.config.rescue': expect_disk_config_rescue,
-            'root': {'bus': 'virtio', 'dev': 'vda',
-                     'type': 'disk', 'boot_index': '1'},
-            }
+            'root': {
+                'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk', 'boot_index': '1',
+            },
+        }
         self.assertEqual(expect, mapping)
 
+    @mock.patch('nova.virt.libvirt.utils.get_arch',
+        new=mock.Mock(return_value=obj_fields.Architecture.X86_64))
+    def test_get_disk_mapping_rescue_with_config__x86_64(self):
+        self._test_get_disk_mapping_rescue_with_config({
+            'bus': 'ide', 'dev': 'hda', 'type': 'cdrom',
+        })
+
+    @mock.patch('nova.virt.libvirt.utils.get_arch',
+        new=mock.Mock(return_value=obj_fields.Architecture.AARCH64))
+    def test_get_disk_mapping_rescue_with_config__aarch64(self):
+        self._test_get_disk_mapping_rescue_with_config({
+            'bus': 'scsi', 'dev': 'sda', 'type': 'cdrom',
+        })
+
     def _test_get_disk_mapping_stable_rescue(
             self, rescue_props, expected, block_device_info, with_local=False):
         instance = objects.Instance(**self.test_instance)
@@ -337,6 +355,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         self._test_get_disk_mapping_stable_rescue(
             rescue_props, expected, block_info)
 
+    @mock.patch('nova.virt.libvirt.utils.get_arch',
+        new=mock.Mock(return_value=obj_fields.Architecture.X86_64))
     def test_get_disk_mapping_stable_rescue_ide_cdrom(self):
         """Assert the disk mapping when rescuing using an IDE cd-rom"""
         rescue_props = {'hw_rescue_device': 'cdrom'}
@@ -478,9 +498,12 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
-        mapping = blockinfo.get_disk_mapping("lxc", instance_ref,
-                                             "lxc", "lxc",
-                                             image_meta)
+        block_device_info = {
+            'image': [{'device_type': 'disk', 'boot_index': 0}],
+        }
+        mapping = blockinfo.get_disk_mapping(
+            "lxc", instance_ref, "lxc", "lxc", image_meta,
+            block_device_info=block_device_info)
         expect = {
             'disk': {'bus': 'lxc', 'dev': None,
                      'type': 'disk', 'boot_index': '1'},
@@ -515,9 +538,14 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref.flavor.swap = 5
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
-        mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
-                                             "virtio", "ide",
-                                             image_meta)
+        block_device_info = {
+            'image': [
+                {'device_type': 'disk', 'boot_index': 0},
+            ]
+        }
+        mapping = blockinfo.get_disk_mapping(
+            "kvm", instance_ref, "virtio", "ide", image_meta,
+            block_device_info=block_device_info)
 
         expect = {
             'disk': {'bus': 'virtio', 'dev': 'vda',
@@ -537,6 +565,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref.ephemeral_gb = 0
 
         block_dev_info = {'swap': None, 'root_device_name': u'/dev/vda',
+            'image': [],
             'ephemerals': [],
             'block_device_mapping': [{'boot_index': None,
                                       'mount_device': u'/dev/vdb',
@@ -549,7 +578,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                       'disk_bus': u'virtio',
                                       'device_type': u'disk'}]}
         instance_ref.flavor.swap = 5
-        image_meta = {}
+        image_meta = objects.ImageMeta.from_dict(None)
 
         mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
                                              "virtio", "ide",
@@ -566,7 +595,9 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             }
         self.assertEqual(expect, mapping)
 
-    def test_get_disk_mapping_simple_configdrive(self):
+    def _test_get_disk_mapping_simple_configdrive(
+        self, expect_bus, expect_dev,
+    ):
         # A simple disk mapping setup, but with configdrive added
         # It's necessary to check if the architecture is power, because
         # power doesn't have support to ide, and so libvirt translate
@@ -577,34 +608,57 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
-        mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
-                                             "virtio", "ide",
-                                             image_meta)
+        block_device_info = {
+            'image': [
+                {'device_type': 'disk', 'boot_index': 0},
+            ]
+        }
+        mapping = blockinfo.get_disk_mapping(
+            "kvm", instance_ref, "virtio", "ide", image_meta,
+            block_device_info=block_device_info)
 
         # Pick the first drive letter on the bus that is available
         # as the config drive. Delete the last device hardcode as
         # the config drive here.
 
-        bus_ppc = ("scsi", "sda")
-        bus_aarch64 = ("scsi", "sda")
-        expect_bus = {"ppc": bus_ppc, "ppc64": bus_ppc,
-                        "ppc64le": bus_ppc, "aarch64": bus_aarch64}
-
-        bus, dev = expect_bus.get(blockinfo.libvirt_utils.get_arch({}),
-                                  ("ide", "hda"))
-
         expect = {
-            'disk': {'bus': 'virtio', 'dev': 'vda',
-                     'type': 'disk', 'boot_index': '1'},
+            'disk': {
+                'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk', 'boot_index': '1',
+            },
             'disk.local': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
-            'disk.config': {'bus': bus, 'dev': dev, 'type': 'cdrom'},
-            'root': {'bus': 'virtio', 'dev': 'vda',
-                     'type': 'disk', 'boot_index': '1'}
-            }
-
+            'disk.config': {
+                'bus': expect_bus, 'dev': expect_dev, 'type': 'cdrom',
+            },
+            'root': {
+                'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk', 'boot_index': '1',
+            },
+        }
         self.assertEqual(expect, mapping)
 
-    def test_get_disk_mapping_cdrom_configdrive(self):
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.X86_64)
+    def test_get_disk_mapping_simple_configdrive__x86_64(self, mock_get_arch):
+        self._test_get_disk_mapping_simple_configdrive('ide', 'hda')
+        mock_get_arch.assert_called()
+
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.PPC64)
+    def test_get_disk_mapping_simple_configdrive__ppc64(self, mock_get_arch):
+        self._test_get_disk_mapping_simple_configdrive('scsi', 'sda')
+        mock_get_arch.assert_called()
+
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.AARCH64)
+    def test_get_disk_mapping_simple_configdrive__aarch64(self, mock_get_arch):
+        self._test_get_disk_mapping_simple_configdrive('scsi', 'sda')
+        mock_get_arch.assert_called()
+
+    def _test_get_disk_mapping_cdrom_configdrive(self, expect_bus, expect_dev):
         # A simple disk mapping setup, with configdrive added as cdrom
         # It's necessary to check if the architecture is power, because
         # power doesn't have support to ide, and so libvirt translate
@@ -616,29 +670,53 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
-        mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
-                                             "virtio", "ide",
-                                             image_meta)
-
-        bus_ppc = ("scsi", "sda")
-        bus_aarch64 = ("scsi", "sda")
-        expect_bus = {"ppc": bus_ppc, "ppc64": bus_ppc,
-                        "ppc64le": bus_ppc, "aarch64": bus_aarch64}
-
-        bus, dev = expect_bus.get(blockinfo.libvirt_utils.get_arch({}),
-                                  ("ide", "hda"))
+        block_device_info = {
+            'image': [
+                {'device_type': 'disk', 'boot_index': 0},
+            ]
+        }
+        mapping = blockinfo.get_disk_mapping(
+            "kvm", instance_ref, "virtio", "ide", image_meta,
+            block_device_info=block_device_info)
 
         expect = {
-            'disk': {'bus': 'virtio', 'dev': 'vda',
-                     'type': 'disk', 'boot_index': '1'},
+            'disk': {
+                'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk', 'boot_index': '1',
+            },
             'disk.local': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
-            'disk.config': {'bus': bus, 'dev': dev, 'type': 'cdrom'},
-            'root': {'bus': 'virtio', 'dev': 'vda',
-                     'type': 'disk', 'boot_index': '1'}
+            'disk.config': {
+                'bus': expect_bus, 'dev': expect_dev, 'type': 'cdrom',
+            },
+            'root': {
+                'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk', 'boot_index': '1',
+            }
             }
 
         self.assertEqual(expect, mapping)
 
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.X86_64)
+    def test_get_disk_mapping_cdrom_configdrive__x86_64(self, mock_get_arch):
+        self._test_get_disk_mapping_cdrom_configdrive('ide', 'hda')
+        mock_get_arch.assert_called()
+
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.PPC64)
+    def test_get_disk_mapping_cdrom_configdrive__ppc64(self, mock_get_arch):
+        self._test_get_disk_mapping_cdrom_configdrive('scsi', 'sda')
+        mock_get_arch.assert_called()
+
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.AARCH64)
+    def test_get_disk_mapping_cdrom_configdrive__aarch64(self, mock_get_arch):
+        self._test_get_disk_mapping_cdrom_configdrive('scsi', 'sda')
+        mock_get_arch.assert_called()
+
     def test_get_disk_mapping_disk_configdrive(self):
         # A simple disk mapping setup, with configdrive added as disk
 
@@ -648,9 +726,14 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
-        mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
-                                             "virtio", "ide",
-                                             image_meta)
+        block_device_info = {
+            'image': [
+                {'device_type': 'disk', 'boot_index': 0},
+            ]
+        }
+        mapping = blockinfo.get_disk_mapping(
+            "kvm", instance_ref, "virtio", "ide", image_meta,
+            block_device_info=block_device_info)
 
         expect = {
             'disk': {'bus': 'virtio', 'dev': 'vda',
@@ -669,6 +752,9 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
         block_device_info = {
+            'image': [
+                {'device_type': 'disk', 'boot_index': 0},
+            ],
             'ephemerals': [
                 {'device_type': 'disk', 'guest_format': 'ext4',
                  'device_name': '/dev/vdb', 'size': 10},
@@ -705,6 +791,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         block_device_info = {
             'swap': {'device_name': '/dev/vdb',
                      'swap_size': 10},
+            'image': [{'device_type': 'disk',
+                       'boot_index': 0}],
             }
         mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
                                              "virtio", "ide",
@@ -726,6 +814,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
         block_device_info = {
+            'image': [],
             'block_device_mapping': [
                 {'connection_info': "fake",
                  'mount_device': "/dev/vda",
@@ -751,9 +840,10 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
     def test_get_disk_mapping_blockdev_root_on_spawn(self):
         # A disk mapping with a blockdev initializing the default root
         instance_ref = objects.Instance(**self.test_instance)
-        image_meta = {}
+        image_meta = objects.ImageMeta.from_dict(None)
 
         block_device_info = {
+            'image': [],
             'block_device_mapping': [
                 {'connection_info': None,
                  'mount_device': None,
@@ -809,6 +899,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
         block_device_info = {
+            'image': [],
             'block_device_mapping': [
                 {'connection_info': "fake",
                  'mount_device': "/dev/vda",
@@ -850,6 +941,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             'root_device_name': '/dev/vdf',
             'swap': {'device_name': '/dev/vdy',
                      'swap_size': 10},
+            'image': [{'device_type': 'disk', 'boot_index': 0}],
             'ephemerals': [
                 {'device_type': 'disk', 'guest_format': 'ext4',
                  'device_name': '/dev/vdb', 'size': 10},
@@ -891,6 +983,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             'swap': {'device_name': '/dev/vdb',
                      'device_type': 'really_lame_type',
                      'swap_size': 10},
+            'image': [{'device_name': '/dev/vda',
+                      'device_type': 'disk'}],
             'ephemerals': [{'disk_bus': 'no_such_bus',
                             'device_type': 'yeah_right',
                             'device_name': '/dev/vdc', 'size': 10}],
@@ -902,6 +996,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             }
         expected_swap = {'device_name': '/dev/vdb', 'disk_bus': 'virtio',
                          'device_type': 'disk', 'swap_size': 10}
+        expected_image = {'device_name': '/dev/vda', 'device_type': 'disk',
+                          'disk_bus': 'virtio'}
         expected_ephemeral = {'disk_bus': 'virtio',
                               'device_type': 'disk',
                               'device_name': '/dev/vdc', 'size': 10}
@@ -921,6 +1017,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         self.assertFalse(get_flavor_mock.called)
 
         self.assertEqual(expected_swap, block_device_info['swap'])
+        self.assertEqual(expected_image, block_device_info['image'][0])
         self.assertEqual(expected_ephemeral,
                          block_device_info['ephemerals'][0])
         self.assertEqual(expected_bdm,
@@ -996,14 +1093,11 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
 
     def test_success_get_disk_bus_for_disk_dev(self):
         expected = (
-                ('ide', ("kvm", "hda")),
-                ('scsi', ("kvm", "sdf")),
-                ('virtio', ("kvm", "vds")),
-                ('fdc', ("kvm", "fdc")),
-                ('uml', ("kvm", "ubd")),
-                ('xen', ("xen", "sdf")),
-                ('xen', ("xen", "xvdb"))
-                )
+            ('ide', ("kvm", "hda")),
+            ('scsi', ("kvm", "sdf")),
+            ('virtio', ("kvm", "vds")),
+            ('fdc', ("kvm", "fdc")),
+        )
         for res, args in expected:
             self.assertEqual(res, blockinfo.get_disk_bus_for_disk_dev(*args))
 
@@ -1078,7 +1172,10 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                  'device_type': 'lame_type',
                  'delete_on_termination': True},
                 {'disk_bus': 'sata', 'guest_format': None,
-                 'device_name': '/dev/sda', 'size': 3}]
+                 'device_name': '/dev/sda', 'size': 3},
+                {'encrypted': True, 'encryption_secret_uuid': uuids.secret,
+                 'encryption_format': 'luks',
+                 'encryption_options': '{"json": "options"}'}]
         expected = [{'dev': 'vds', 'type': 'disk', 'bus': 'usb'},
                     {'dev': 'vdb', 'type': 'disk',
                      'bus': 'virtio', 'format': 'ext4'},
@@ -1087,7 +1184,11 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                      'bus': 'scsi', 'boot_index': '1'},
                     {'dev': 'vdo', 'type': 'disk',
                      'bus': 'scsi', 'boot_index': '2'},
-                    {'dev': 'sda', 'type': 'disk', 'bus': 'sata'}]
+                    {'dev': 'sda', 'type': 'disk', 'bus': 'sata'},
+                    {'dev': 'vda', 'type': 'disk', 'bus': 'virtio',
+                     'encrypted': True, 'encryption_secret_uuid': uuids.secret,
+                     'encryption_format': 'luks',
+                     'encryption_options': {'json': 'options'}}]
 
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
         for bdm, expected in zip(bdms, expected):
@@ -1186,8 +1287,21 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
 
         self.assertEqual('virtio', info['bus'])
 
+    @mock.patch('nova.virt.libvirt.blockinfo.get_info_from_bdm')
+    def test_get_root_info_bdm_with_iso_image(self, mock_get_info):
+        self.test_image_meta['disk_format'] = 'iso'
+        instance = objects.Instance(**self.test_instance)
+        image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
+        init_root_bdm = {'device_type': 'disk'}
+        iso_root_bdm = {'device_type': 'cdrom', 'disk_bus': 'ide'}
+        blockinfo.get_root_info(instance, 'kvm', image_meta, init_root_bdm,
+                                'virtio', 'ide')
+        mock_get_info.assert_called_once_with(instance, 'kvm', image_meta,
+                                              iso_root_bdm, {}, 'virtio')
+
     @mock.patch('nova.virt.libvirt.blockinfo.get_info_from_bdm')
     def test_get_root_info_bdm(self, mock_get_info):
+        # call get_root_info() with DriverBlockDevice
         instance = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
         root_bdm = {'mount_device': '/dev/vda',
@@ -1216,15 +1330,45 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                                'device_type': 'disk'},
                                               {}, 'virtio')
         mock_get_info.reset_mock()
-        # xen with incompatible root_device_name/disk_bus combination
-        root_bdm['disk_bus'] = 'xen'
-        blockinfo.get_root_info(instance, 'xen', image_meta, root_bdm,
-                                'xen', 'ide', root_device_name='sda')
-        mock_get_info.assert_called_once_with(instance, 'xen', image_meta,
-                                              {'device_name': 'xvda',
-                                               'disk_bus': 'xen',
-                                               'device_type': 'disk'},
-                                              {}, 'xen')
+
+    @mock.patch('nova.virt.libvirt.blockinfo.get_info_from_bdm')
+    def test_get_root_info_bdm_with_deepcopy(self, mock_get_info):
+        # call get_root_info() with BlockDeviceMapping
+        instance = objects.Instance(**self.test_instance)
+        image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
+        root_bdm = objects.BlockDeviceMapping(self.context,
+            **fake_block_device.FakeDbBlockDeviceDict(
+                {'id': 3, 'instance_uuid': uuids.instance,
+                 'device_name': '/dev/sda',
+                 'source_type': 'blank',
+                 'destination_type': 'local',
+                 'device_type': 'cdrom',
+                 'disk_bus': 'virtio',
+                 'volume_id': 'fake-volume-id-1',
+                 'boot_index': 0}))
+        # No root_device_name
+        blockinfo.get_root_info(
+            instance, 'kvm', image_meta, root_bdm, 'virtio', 'ide')
+        mock_get_info.reset_mock()
+        # Both device names
+        blockinfo.get_root_info(
+            instance, 'kvm', image_meta, root_bdm, 'virtio', 'scsi',
+            root_device_name='/dev/sda')
+        mock_get_info.reset_mock()
+        # Missing device names
+        original_bdm = copy.deepcopy(root_bdm)
+        root_bdm.device_name = ''
+        blockinfo.get_root_info(
+            instance, 'kvm', image_meta, root_bdm, 'virtio', 'scsi',
+            root_device_name='/dev/sda')
+        mock_get_info.assert_called_with(
+            instance, 'kvm', image_meta, mock.ANY, {}, 'virtio')
+        actual_call = mock_get_info.call_args
+        _, _, _, actual_bdm, _, _ = actual_call[0]
+        self.assertEqual(
+            original_bdm.obj_to_primitive(),
+            actual_bdm.obj_to_primitive()
+        )
 
     def test_get_boot_order_simple(self):
         disk_info = {
@@ -1302,8 +1446,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
 
     def test_get_rescue_bus(self):
         # Assert that all supported device bus types are returned. Stable
-        # device rescue is not supported by xen or lxc so ignore these.
-        for virt_type in ['qemu', 'kvm', 'uml', 'parallels']:
+        # device rescue is not supported by lxc so ignore this.
+        for virt_type in ['qemu', 'kvm', 'parallels']:
             for bus in blockinfo.SUPPORTED_DEVICE_BUSES[virt_type]:
                 meta = self._get_rescue_image_meta({'hw_rescue_bus': bus})
                 self.assertEqual(bus, blockinfo.get_rescue_bus(None, virt_type,
@@ -1319,30 +1463,29 @@ class DefaultDeviceNamesTestCase(test.NoDBTestCase):
     def setUp(self):
         super(DefaultDeviceNamesTestCase, self).setUp()
         self.context = context.get_admin_context()
+        self.flavor = objects.Flavor(id=2, swap=4)
         self.instance = objects.Instance(
-                uuid='32dfcb37-5af1-552b-357c-be8c3aa38310',
-                memory_kb='1024000',
-                basepath='/some/path',
-                bridge_name='br100',
-                vcpus=2,
-                project_id='fake',
-                bridge='br101',
-                image_ref='155d900f-4e14-4e4c-a73d-069cbf4541e6',
-                root_gb=10,
-                ephemeral_gb=20,
-                instance_type_id=2,
-                config_drive=False,
-                root_device_name = '/dev/vda',
-                system_metadata={})
+            uuid='32dfcb37-5af1-552b-357c-be8c3aa38310',
+            memory_kb='1024000',
+            basepath='/some/path',
+            bridge_name='br100',
+            vcpus=2,
+            project_id='fake',
+            bridge='br101',
+            image_ref='155d900f-4e14-4e4c-a73d-069cbf4541e6',
+            root_gb=10,
+            ephemeral_gb=20,
+            instance_type_id=self.flavor.id,
+            flavor=self.flavor,
+            config_drive=False,
+            root_device_name = '/dev/vda',
+            system_metadata={})
         self.image_meta = objects.ImageMeta(
             disk_format='raw',
             properties=objects.ImageMetaProps())
 
         self.virt_type = 'kvm'
-        self.flavor = objects.Flavor(swap=4)
         self.patchers = []
-        self.patchers.append(mock.patch.object(self.instance, 'get_flavor',
-                                               return_value=self.flavor))
         self.patchers.append(mock.patch(
                 'nova.objects.block_device.BlockDeviceMapping.save'))
         for patcher in self.patchers:
@@ -1405,13 +1548,25 @@ class DefaultDeviceNamesTestCase(test.NoDBTestCase):
                  'destination_type': 'volume',
                  'boot_index': -1}))]
 
+        self.image = [
+            objects.BlockDeviceMapping(self.context,
+                **fake_block_device.FakeDbBlockDeviceDict(
+                {'id': 6, 'instance_uuid': uuids.instance,
+                 'source_type': 'image',
+                 'destination_type': 'local',
+                 'device_type': 'disk',
+                 'boot_index': 0}))]
+
     def tearDown(self):
         super(DefaultDeviceNamesTestCase, self).tearDown()
         for patcher in self.patchers:
             patcher.stop()
 
-    def _test_default_device_names(self, eph, swap, bdm):
-        bdms = eph + swap + bdm
+    @mock.patch(
+        'nova.virt.libvirt.utils.get_arch',
+        return_value=obj_fields.Architecture.X86_64)
+    def _test_default_device_names(self, eph, swap, bdm, mock_get_arch):
+        bdms = self.image + eph + swap + bdm
         bdi = driver.get_block_device_info(self.instance, bdms)
         blockinfo.default_device_names(self.virt_type,
                                        self.context,
@@ -1419,6 +1574,8 @@ class DefaultDeviceNamesTestCase(test.NoDBTestCase):
                                        bdi,
                                        self.image_meta)
 
+        mock_get_arch.assert_called()
+
     def test_only_block_device_mapping(self):
         # Test no-op
         original_bdm = copy.deepcopy(self.block_device_mapping)
