@@ -462,6 +462,8 @@ class HostTestCase(test.NoDBTestCase):
         h.initialize()
 
         h.get_connection()
+        h._dispatch_conn_event()
+
         event.wait()
         # This test will timeout if it fails. Success is implicit in a
         # timely return from wait(), indicating that the connection event
@@ -1052,6 +1054,12 @@ Active:          8381604 kB
              'iowait': 6121490000000},
             stats)
 
+    @mock.patch.object(fakelibvirt.virConnect, "getCPUMap")
+    def test_get_available_cpus(self, mock_map):
+        mock_map.return_value = (4, [True, True, False, False], None)
+        result = self.host.get_available_cpus()
+        self.assertEqual(result, {0, 1, 2, 3})
+
     @mock.patch.object(fakelibvirt.virConnect, "defineXML")
     def test_write_instance_config(self, mock_defineXML):
         fake_dom_xml = """
@@ -2146,6 +2154,48 @@ class LibvirtTpoolProxyTestCase(test.NoDBTestCase):
             self.assertIsInstance(domain, tpool.Proxy)
             self.assertIn(domain.UUIDString(), (uuids.vm1, uuids.vm2))
 
+    def _add_fake_host_devices(self):
+        self.conn._obj.pci_info = fakelibvirt.HostPCIDevicesInfo(
+            num_pci=1, num_pfs=2, num_vfs=2, num_mdevcap=3)
+        mdevs = {
+            'mdev_4b20d080_1b54_4048_85b3_a6a62d165c01':
+                fakelibvirt.FakeMdevDevice(
+                    dev_name='mdev_4b20d080_1b54_4048_85b3_a6a62d165c01',
+                    type_id=fakelibvirt.NVIDIA_11_VGPU_TYPE,
+                    parent=fakelibvirt.MDEVCAP_DEV1_PCI_ADDR),
+        }
+        self.conn._obj.mdev_info = fakelibvirt.HostMdevDevicesInfo(
+            devices=mdevs)
+
+    def test_tpool_list_all_devices(self):
+        self._add_fake_host_devices()
+        devs = self.host.list_all_devices(
+            fakelibvirt.VIR_CONNECT_LIST_NODE_DEVICES_CAP_PCI_DEV)
+        self.assertEqual(8, len(devs))
+        for dev in devs:
+            self.assertIsInstance(dev, tpool.Proxy)
+
+    def test_tpool_list_pci_devices(self):
+        self._add_fake_host_devices()
+        dev_names = self.host.list_pci_devices()
+        self.assertEqual(8, len(dev_names))
+        for name in dev_names:
+            self.assertIsInstance(name, str)
+
+    def test_tpool_list_mdev_capable_devices(self):
+        self._add_fake_host_devices()
+        dev_names = self.host.list_mdev_capable_devices()
+        self.assertEqual(3, len(dev_names))
+        for name in dev_names:
+            self.assertIsInstance(name, str)
+
+    def test_tpool_list_mediated_devices(self):
+        self._add_fake_host_devices()
+        dev_names = self.host.list_mediated_devices()
+        self.assertEqual(1, len(dev_names))
+        for name in dev_names:
+            self.assertIsInstance(name, str)
+
 
 class LoadersTestCase(test.NoDBTestCase):
 
