@@ -66,6 +66,7 @@ from nova.virt.libvirt import event as libvirtevent
 from nova.virt.libvirt import guest as libvirt_guest
 from nova.virt.libvirt import migration as libvirt_migrate
 from nova.virt.libvirt import utils as libvirt_utils
+import nova.virt.node  # noqa
 
 if ty.TYPE_CHECKING:
     import libvirt
@@ -138,6 +139,7 @@ class Host(object):
         self._caps = None
         self._domain_caps = None
         self._hostname = None
+        self._node_uuid = None
 
         self._wrapped_conn = None
         self._wrapped_conn_lock = threading.Lock()
@@ -496,6 +498,9 @@ class Host(object):
         LOG.debug("Starting green dispatch thread")
         utils.spawn(self._dispatch_thread)
 
+        LOG.debug("Starting connection event dispatch thread")
+        utils.spawn(self._conn_event_thread)
+
     def _get_new_connection(self):
         # call with _wrapped_conn_lock held
         LOG.debug('Connecting to libvirt: %s', self._uri)
@@ -615,9 +620,6 @@ class Host(object):
         libvirt.virEventRegisterDefaultImpl()
         self._init_events()
 
-        LOG.debug("Starting connection event dispatch thread")
-        utils.spawn(self._conn_event_thread)
-
         self._initialized = True
 
     def _version_check(self, lv_ver=None, hv_ver=None, hv_type=None,
@@ -738,6 +740,14 @@ class Host(object):
 
         return doms
 
+    def get_available_cpus(self):
+        """Get the set of CPUs that exist on the host.
+
+        :returns: set of CPUs, raises libvirtError on error
+        """
+        cpus, cpu_map, online = self.get_connection().getCPUMap()
+        return {cpu for cpu in range(cpus)}
+
     def get_online_cpus(self):
         """Get the set of CPUs that are online on the host
 
@@ -1059,6 +1069,12 @@ class Host(object):
                       {'old': self._hostname, 'new': hostname})
         return self._hostname
 
+    def get_node_uuid(self):
+        """Returns the UUID of this node."""
+        if not self._node_uuid:
+            self._node_uuid = nova.virt.node.get_local_node_uuid()
+        return self._node_uuid
+
     def find_secret(self, usage_type, usage_id):
         """Find a secret.
 
@@ -1549,31 +1565,31 @@ class Host(object):
         nodedev = self.get_vdpa_nodedev_by_address(pci_address)
         return nodedev.vdpa_capability.dev_path
 
-    def list_pci_devices(self, flags=0):
+    def list_pci_devices(self, flags: int = 0) -> ty.List[str]:
         """Lookup pci devices.
 
-        :returns: a list of virNodeDevice instance
+        :returns: a list of strings, names of the virNodeDevice instances
         """
         return self._list_devices("pci", flags=flags)
 
-    def list_mdev_capable_devices(self, flags=0):
+    def list_mdev_capable_devices(self, flags: int = 0) -> ty.List[str]:
         """Lookup devices supporting mdev capabilities.
 
-        :returns: a list of virNodeDevice instance
+        :returns: a list of strings, names of the virNodeDevice instances
         """
         return self._list_devices("mdev_types", flags=flags)
 
-    def list_mediated_devices(self, flags=0):
+    def list_mediated_devices(self, flags: int = 0) -> ty.List[str]:
         """Lookup mediated devices.
 
-        :returns: a list of strings with the name of the instance
+        :returns: a list of strings, names of the virNodeDevice instances
         """
         return self._list_devices("mdev", flags=flags)
 
-    def _list_devices(self, cap, flags=0):
+    def _list_devices(self, cap, flags: int = 0) -> ty.List[str]:
         """Lookup devices.
 
-        :returns: a list of virNodeDevice instance
+        :returns: a list of strings, names of the virNodeDevice instances
         """
         try:
             return self.get_connection().listDevices(cap, flags)
@@ -1596,7 +1612,10 @@ class Host(object):
         :returns: a list of virNodeDevice xml strings.
         """
         try:
-            return self.get_connection().listAllDevices(flags) or []
+            alldevs = [
+                self._wrap_libvirt_proxy(dev)
+                for dev in self.get_connection().listAllDevices(flags)] or []
+            return alldevs
         except libvirt.libvirtError as ex:
             LOG.warning(ex)
             return []
@@ -1605,6 +1624,22 @@ class Host(object):
         """Compares the given CPU description with the host CPU."""
         return self.get_connection().compareCPU(xmlDesc, flags)
 
+    def compare_hypervisor_cpu(self, xmlDesc, flags=0):
+        """Compares the given CPU description with the CPU provided by
+        the host hypervisor. This is different from the older method,
+        compare_cpu(), which compares a given CPU definition with the
+        host CPU without considering the abilities of the host
+        hypervisor. Except @xmlDesc, rest of all the parameters to
+        compareHypervisorCPU API are optional (libvirt will choose
+        sensible defaults).
+        """
+        emulator = None
+        arch = None
+        machine = None
+        virttype = None
+        return self.get_connection().compareHypervisorCPU(
+            emulator, arch, machine, virttype, xmlDesc, flags)
+
     def is_cpu_control_policy_capable(self):
         """Returns whether kernel configuration CGROUP_SCHED is enabled
 
